from nsbaseresource import NSBaseResource
__author__ = 'vlazarenko'


class NSLBMonitorServiceBinding(NSBaseResource):

    # Binding monitors to lb vservers

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSLBMonitorServiceBinding, self).__init__()
        self.options = {'monitorname': '',
                        'servicename': '',
                        'dup_state': '',
                        'dup_weight': '',
                        'servicegroupname': '',
                        'state': '',
                        'weight': ''}

        self.resourcetype = NSLBMonitorServiceBinding.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "lbmonitor_service_binding"

    def set_monitorname(self, monitorname):
        self.options['monitorname'] = monitorname

    def get_monitorname(self):
        return self.options['monitorname']

    def set_servicename(self, servicename):
        self.options['servicename'] = servicename

    def get_servicename(self):
        return self.options['servicename']

    def set_dup_state(self, dup_state):
        self.options['dup_state'] = dup_state

    def get_dup_state(self):
        return self.options['dup_state']

    def set_dup_weight(self, dup_weight):
        self.options['dup_weight'] = dup_weight

    def get_dup_weight(self):
        return self.options['dup_weight']

    def set_servicegroupname(self, servicegroupname):
        self.options['servicegroupname'] = servicegroupname

    def get_servicegroupname(self):
        return self.options['servicegroupname']

    def set_state(self, state):
        self.options['state'] = state

    def get_state(self):
        return self.options['state']

    def set_weight(self, weight):
        self.options['weight'] = weight

    def get_weight(self):
        return self.options['weight']

    @staticmethod
    def add(nitro, resource):
        """
        Use this API to add lbmonitor_service_binding.
        """
        __resource = NSLBMonitorServiceBinding()
        __resource.set_monitorname(resource.get_monitorname())
        __resource.set_servicename(resource.get_servicename())
        __resource.set_dup_state(resource.get_dup_state())
        __resource.set_dup_weight(resource.get_dup_weight())
        __resource.set_servicegroupname(resource.get_servicegroupname())
        __resource.set_state(resource.get_state())
        __resource.set_weight(resource.get_weight())
        return __resource.update_resource(nitro)

    @staticmethod
    def delete(nitro, resource):
        """
        Use this API to delete lbmonitor_service_binding of a given name.
        """
        __resource = NSLBMonitorServiceBinding()
        __resource.set_monitorname(resource.get_monitorname())
        __resource.set_servicename(resource.get_servicename())
        __resource.set_servicegroupname(resource.get_servicegroupname())
        nsresponse = __resource.delete_resource(nitro, object_name=__resource.get_monitorname())
        return nsresponse



from nsbaseresource import NSBaseResource

__author__ = 'timl'


class NSResponderAction(NSBaseResource):

    # Configuration for responder actions.

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSResponderAction, self).__init__()
        self.options = {
            'name': '',
            'newname': '',
            'type': '',
            'target': '',
            'bypasssafetycheck': '',
            'hits': '',
            'referencecount': '',
            'undefhits': '',
        }

        self.resourcetype = NSResponderAction.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "responderaction"

    def set_name(self, name):
        """
        Name of the responder action.

        Default value: 0
        """
        self.options['name'] = name

    def get_name(self):
        """
        Name of the responder action.

        Default value: 0
        """
        return self.options['name']

    def set_type(self, action_type):
        """
        Type (respondwith, redirect) of responder action.

        Default value: 0
        """
        self.options['type'] = action_type

    def get_type(self):
        """
        Type (respondwith, redirect) of responder action.

        Default value: 0
        """
        return self.options['type']

    def set_target(self, target):
        """
        Target of responder action.

        Default value: 0
        """
        self.options['target'] = target

    def get_target(self):
        """
        Target of responder action.

        Default value: 0
        """
        return self.options['target']

    def set_newname(self, newname):
        """
        The new name of the responder action.

        Default value: 0
        Minimum length =  1.
        """
        self.options['newname'] = newname

    def get_newname(self):
        """
        The new name of the responder action.

        Default value: 0
        Minimum length =  1.
        """
        return self.options['newname']

    def get_hits(self):
        """
        Number of hits.

        Default value: 0
        """
        return self.options['hits']

    def get_undefhits(self):
        """
        Number of undef hits.

        Default value: 0
        """
        return self.options['undefhits']

    def get_referencecount(self):
        """
        Number of references to this action.

        Default value: 0
        """
        return self.options['referencecount']

    def set_bypasssafetycheck(self, bypasssafetycheck):
        """
        Bypass the safety check

        Default value: NO
        """
        valid = ('YES', 'NO')
        if bypasssafetycheck and bypasssafetycheck not in valid:
            raise ValueError("bypasssafetycheck must be one of %s" %
                             ",".join(valid))
        self.options['bypasssafetycheck'] = bypasssafetycheck

    def get_bypasssafetycheck(self):
        """
        Bypass the safety check

        Default value: NO
        """
        return self.options['bypasssafetycheck']

    # Operations methods
    @staticmethod
    def get(nitro, responderaction):
        """
        Use this API to fetch responderaction resource of given name.
        """
        __responderaction = NSResponderAction()
        __responderaction.set_name(responderaction.get_name())
        __responderaction.get_resource(nitro)
        return __responderaction

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured responderaction resources.
        """
        __url = nitro.get_url() + NSResponderAction.get_resourcetype()
        __json_cspolicies = nitro.get(__url).get_response_field(NSResponderAction.get_resourcetype())
        __responderpolicies = []
        for json_responderaction in __json_cspolicies:
            __responderpolicies.append(NSResponderAction(json_responderaction))
        return __responderpolicies

    @staticmethod
    def add(nitro, responderaction):
        """
        Use this API to add responderaction.
        """
        __responderaction = NSResponderAction()
        __responderaction.set_name(responderaction.get_name())
        __responderaction.set_type(responderaction.get_type())
        __responderaction.set_target(responderaction.get_target())
        __responderaction.set_bypasssafetycheck(responderaction.get_bypasssafetycheck())
        return __responderaction.add_resource(nitro)

    @staticmethod
    def delete(nitro, responderaction):
        """
        Use this API to delete responderaction of a given name.
        """
        __responderaction = NSResponderAction()
        __responderaction.set_name(responderaction.get_name())
        nsresponse = __responderaction.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, responderaction):
        """
        Use this API to update a responderaction of a given name.
        """
        __responderaction = NSResponderAction()
        __responderaction.set_name(responderaction.get_name())
        __responderaction.set_target(responderaction.get_target())
        __responderaction.set_bypasssafetycheck(responderaction.get_bypasssafetycheck())
        return __responderaction.update_resource(nitro)

    # No unset functionality for now.
    @staticmethod
    def rename(nitro, responderaction):
        """
        Use this API to rename responderaction.
        """
        __responderaction = NSResponderAction()
        __responderaction.set_name(responderaction.get_name())
        __responderaction.set_newname(responderaction.get_newname())
        return __responderaction.perform_operation(nitro, "rename")



from nsbaseresource import NSBaseResource

__author__ = 'ndenev@gmail.com'


class AuthTacacsPolicy(NSBaseResource):

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(AuthTacacsPolicy, self).__init__()

        self.options = {'name': '',
                        'rule': '',
                        'reqaction': ''}

        self.resourcetype = AuthTacacsPolicy.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "authenticationtacacspolicy"

    def set_name(self, name):
        """
        Name for the TACACS+ policy. Must begin with a letter, number, or the underscore character (_), and must contain only letters,
        numbers, and the hyphen (-), period (.) pound (#), space ( ), at (@), equals (=), colon (:), and underscore characters.
        Cannot be changed after TACACS+ policy is created.
        Minimum length = 1
        """
        self.options['name'] = name

    def get_name(self):
        return self.options['name']

    def set_rule(self, rule):
        """
        Name of the NetScaler named rule, or a default syntax expression, that the policy uses to determine whether to attempt to authenticate the user with the TACACS+ server.
        Minimum length = 1
        """
        self.options['rule'] = rule

    def get_rule(self):
        return self.options['rule']

    def set_reqaction(self, reqaction):
        """
        Name of the TACACS+ action to perform if the policy matches.
        Minimum length = 1
        """
        self.options['reqaction'] = reqaction

    def get_reqaction(self):
        return self.options['reqaction']

    # Operations methods
    @staticmethod
    def get(nitro, tacacspolicy):
        """
        Use this API to fetch a configured authenticationtacacspolicy resource by name.
        """
        __tacacspolicy = AuthTacacsPolicy()
        __tacacspolicy.set_name(tacacspolicy.get_name())
        __tacacspolicy.get_resource(nitro)
        return __tacacspolicy

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured authenticationtacacspolicy resources.
        """
        __url = nitro.get_url() + AuthTacacsPolicy.get_resourcetype()
        __json_tacacspolicies = nitro.get(__url).get_response_field(AuthTacacsPolicy.get_resourcetype())
        __tacacspolicies = []
        for json_tacacspolicy in __json_tacacspolicies:
            __tacacspolicies.append(AuthTacacsPolicy(json_tacacspolicy))
        return __tacacspolicies

    @staticmethod
    def add(nitro, tacacspolicy):
        """
        Use this API to add authenticationtacacspolicy.
        """
        __tacacspolicy = AuthTacacsPolicy()
        __tacacspolicy.set_name(tacacspolicy.get_name())
        __tacacspolicy.set_rule(tacacspolicy.get_rule())
        __tacacspolicy.set_reqaction(tacacspolicy.get_reqaction())
        return __tacacspolicy.add_resource(nitro)

    @staticmethod
    def delete(nitro, tacacspolicy):
        """
        Use this API to delete authenticationtacacspolicy of a given name.
        """
        __tacacspolicy = AuthTacacsPolicy()
        __tacacspolicy.set_name(tacacspolicy.get_name())
        nsresponse = __tacacspolicy.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, tacacspolicy):
        """
        Use this API to update a rewritepolicy of a given name.
        """
        __tacacspolicy = AuthTacacsPolicy()
        __tacacspolicy.set_name(tacacspolicy.get_name())
        __tacacspolicy.set_rule(tacacspolicy.get_rule())
        __tacacspolicy.set_reqaction(tacacspolicy.get_reqaction())
        return __tacacspolicy.update_resource(nitro)

    # No unset functionality for now.



from nsbaseresource import NSBaseResource
__author__ = 'ndenev'


class NSPbr(NSBaseResource):

    # Configuration for PBR entry resource.

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSPbr, self).__init__()
        self.options = {'name': '',
                        'action': '',
                        'td': '',
                        'srcip': '',
                        'srcipop': '',
                        'srcipval': '',
                        'srcport': '',
                        'srcportop': '',
                        'srcportval': '',
                        'destip': '',
                        'destipop': '',
                        'destipval': '',
                        'destport': '',
                        'destportval': '',
                        'nexthop': '',
                        'nexthopval': '',
                        'iptunnel': '',
                        'iptunnelname': '',
                        'srcmac': '',
                        'protocol': '',
                        'protocolnumber': '',
                        'vlan': '',
                        'Interface': '',
                        'priority': '',
                        'msr': '',
                        'monitor': '',
                        'state': '',
                        'detail': '',
                        'hits': '',
                        'kernelstate': '',
                        'curstate': '',
                        'totalprobes': '',
                        'totalfailedprobes': '',
                        'failedprobes': '',
                        'monstatcode': '',
                        'monstatparam1': '',
                        'monstatparam2': '',
                        'monstatparam3': '',
                        'data': '',
                        '__count': ''
                        }

        self.resourcetype = NSPbr.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "nspbr"

    def set_name(self, name):
        self.options['name'] = name

    def get_name(self):
        return self.options['name']

    def set_action(self, action):
        self.options['action'] = action

    def get_action(self):
        return self.options['action']

    def set_td(self, td):
        self.options['td'] = td

    def get_td(self):
        return self.options['td']

    def set_srcip(self, srcip):
        self.options['srcip'] = srcip

    def get_srcip(self):
        return self.options['srcip']

    def set_srcipop(self, srcipop):
        self.options['srcipop'] = srcipop

    def get_srcipop(self):
        return self.options['srcipop']

    def set_srcipval(self, srcipval):
        self.options['srcipval'] = srcipval

    def get_srcipval(self):
        return self.options['srcipval']

    def set_srcport(self, srcport):
        self.options['srcport'] = srcport

    def get_srcport(self):
        return self.options['srcport']

    def set_srcportop(self, srcportop):
        self.options['srcportop'] = srcportop

    def get_srcportop(self):
        return self.options['srcportop']

    def set_srcportval(self, srcportval):
        self.options['srcportval'] = srcportval

    def get_srcportval(self):
        return self.options['srcportval']

    def set_destip(self, destip):
        self.options['destip'] = destip

    def get_destip(self):
        return self.options['destip']

    def set_destipop(self, destipop):
        self.options['destipop'] = destipop

    def get_destipop(self):
        return self.options['destipop']

    def set_destipval(self, destipval):
        self.options['destipval'] = destipval

    def get_destipval(self):
        return self.options['destipval']

    def set_destport(self, destport):
        self.options['destport'] = destport

    def get_destport(self):
        return self.options['destport']

    def set_destportval(self, destportval):
        self.options['destportval'] = destportval

    def get_destportval(self):
        return self.options['destportval']

    def set_nexthop(self, nexthop):
        self.options['nexthop'] = nexthop

    def get_nexthop(self):
        return self.options['nexthop']

    def set_nexthopval(self, nexthopval):
        self.options['nexthopval'] = nexthopval

    def get_nexthopval(self):
        return self.options['nexthopval']

    def set_iptunnel(self, iptunnel):
        self.options['iptunnel'] = iptunnel

    def get_iptunnel(self):
        return self.options['iptunnel']

    def set_iptunnelname(self, iptunnelname):
        self.options['iptunnelname'] = iptunnelname

    def get_iptunnelname(self):
        return self.options['iptunnelname']

    def set_srcmac(self, srcmac):
        self.options['srcmac'] = srcmac

    def get_srcmac(self):
        return self.options['srcmac']

    def set_protocol(self, protocol):
        self.options['protocol'] = protocol

    def get_protocol(self):
        return self.options['protocol']

    def set_protocolnumber(self, protocolnumber):
        self.options['protocolnumber'] = protocolnumber

    def get_protocolnumber(self):
        return self.options['protocolnumber']

    def set_vlan(self, vlan):
        self.options['vlan'] = vlan

    def get_vlan(self):
        return self.options['vlan']

    def set_interface(self, interface):
        self.options['Interface'] = interface

    def get_interface(self):
        return self.options['Interface']

    def set_priority(self, priority):
        self.options['priority'] = priority

    def get_priority(self):
        return self.options['priority']

    def set_msr(self, msr):
        self.options['msr'] = msr

    def get_msr(self):
        return self.options['msr']

    def set_monitor(self, monitor):
        self.options['monitor'] = monitor

    def get_monitor(self):
        return self.options['monitor']

    def set_state(self, state):
        self.options['state'] = state

    def get_state(self):
        return self.options['state']

    def set_detail(self, detail):
        self.options['detail'] = detail

    def get_detail(self):
        return self.options['detail']

    # ReadOnly
    def get_hits(self):
        return self.options['hits']

    def get_kernelstate(self):
        return self.options['kernelstate']

    def get_curstate(self):
        return self.options['curstate']

    def get_totalprobes(self):
        return self.options['totalprobes']

    def get_totalfailedprobes(self):
        return self.options['totalfailedprobes']

    def get_failedprobes(self):
        return self.options['failedprobes']

    def get_monstatcode(self):
        return self.options['monstatcode']

    def get_monstatparam1(self):
        return self.options['monstatparam1']

    def get_monstatparam2(self):
        return self.options['monstatparam2']

    def get_monstatparam3(self):
        return self.options['monstatparam3']

    def get_data(self):
        return self.options['data']

    # Operations methods

    @staticmethod
    def add(nitro, nspbr):
        """
        Use this API to add NSPbr resources.
        """
        __NSPbr = NSPbr()
        __NSPbr.set_name(nspbr.get_name())
        __NSPbr.set_action(nspbr.get_action())
        __NSPbr.set_td(nspbr.get_td())
        __NSPbr.set_srcip(nspbr.get_srcip())
        __NSPbr.set_srcipop(nspbr.get_srcipop())
        __NSPbr.set_srcipval(nspbr.get_srcipval())
        __NSPbr.set_srcport(nspbr.get_srcport())
        __NSPbr.set_srcportop(nspbr.get_srcportop())
        __NSPbr.set_srcportval(nspbr.get_srcportval())
        __NSPbr.set_destip(nspbr.get_destip())
        __NSPbr.set_destipop(nspbr.get_destipop())
        __NSPbr.set_destipval(nspbr.get_destipval())
        __NSPbr.set_destport(nspbr.get_destport())
        __NSPbr.set_destportop(nspbr.get_destportop())
        __NSPbr.set_destportval(nspbr.get_destportval())
        __NSPbr.set_nexthop(nspbr.get_nexthop())
        __NSPbr.set_nexthopval(nspbr.get_nexthopval())
        __NSPbr.set_iptunnel(nspbr.get_iptunnel())
        __NSPbr.set_iptunnelname(nspbr.get_iptunnelname())
        __NSPbr.set_srcmac(nspbr.get_srcmac())
        __NSPbr.set_protocol(nspbr.get_protocol())
        __NSPbr.set_protocolnumber(nspbr.get_protocolnumber())
        __NSPbr.set_vlan(nspbr.get_vlan())
        __NSPbr.set_Interface(nspbr.get_Interface())
        __NSPbr.set_priority(nspbr.get_priority())
        __NSPbr.set_msr(nspbr.get_msr())
        __NSPbr.set_monitor(nspbr.get_monitor())
        __NSPbr.set_state(nspbr.get_state())
        return __NSPbr.add_resource(nitro)

    @staticmethod
    def delete(nitro, nspbr):
        """
        Use this API to delete NSPbr resources.
        """
        __NSPbr = NSPbr()
        __NSPbr.set_name(nspbr.get_name())
        nsresponse = __NSPbr.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, nspbr):
        """
        Use this API to update NSPbr resources.
        """
        __NSPbr = NSPbr()
        __NSPbr.set_name(nspbr.get_name())
        __NSPbr.set_action(nspbr.get_action())
        __NSPbr.set_srcip(nspbr.get_srcip())
        __NSPbr.set_srcipop(nspbr.get_srcipop())
        __NSPbr.set_srcipval(nspbr.get_srcipval())
        __NSPbr.set_srcport(nspbr.get_srcport())
        __NSPbr.set_srcportop(nspbr.get_srcportop())
        __NSPbr.set_srcportval(nspbr.get_srcportval())
        __NSPbr.set_destip(nspbr.get_destip())
        __NSPbr.set_destipop(nspbr.get_destipop())
        __NSPbr.set_destipval(nspbr.get_destipval())
        __NSPbr.set_destport(nspbr.get_destport())
        __NSPbr.set_destportop(nspbr.get_destportop())
        __NSPbr.set_destportval(nspbr.get_destportval())
        __NSPbr.set_nexthop(nspbr.get_nexthop())
        __NSPbr.set_nexthopval(nspbr.get_nexthopval())
        __NSPbr.set_iptunnel(nspbr.get_iptunnel())
        __NSPbr.set_iptunnelname(nspbr.get_iptunnelname())
        __NSPbr.set_srcmac(nspbr.get_srcmac())
        __NSPbr.set_protocol(nspbr.get_protocol())
        __NSPbr.set_protocolnumber(nspbr.get_protocolnumber())
        __NSPbr.set_vlan(nspbr.get_vlan())
        __NSPbr.set_interface(nspbr.get_interface())
        __NSPbr.set_priority(nspbr.get_priority())
        __NSPbr.set_msr(nspbr.get_msr())
        __NSPbr.set_monitor(nspbr.get_monitor())
        return __NSPbr.update_resource(nitro)

#    @staticmethod
#    def unset(nitro, NSPbr):
#        """ generated source for method unset """
#        unsetresource = NSPbr()
#        unsetresource.name = pbrname
#        return unsetresource.unset_resource(client, args)

    @staticmethod
    def enable(nitro, nspbr):
        """
        Use this API to enable a acl of given name.
        """
        __NSPbr = NSPbr()
        __NSPbr.set_name(nspbr.get_name())
        return __NSPbr.perform_operation(nitro, "enable")

    @staticmethod
    def disable(nitro, nspbr):
        """
        Use this API to disable a acl of given name.
        """
        __NSPbr = NSPbr()
        __NSPbr.set_name(nspbr.get_name())
        return __NSPbr.perform_operation(nitro, "disable")

    @staticmethod
    def get(nitro, nspbr):
        """
        Use this API to fetch NSPbr resource of given name.
        """
        __NSPbr = NSPbr()
        __NSPbr.set_name(nspbr.get_name())
        __NSPbr.get_resource(nitro, object_name=__NSPbr.get_name())
        return __NSPbr

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured NSPbr resources.
        """
        __url = nitro.get_url() + NSPbr.get_resourcetype()
        __json_NSPbrs = nitro.get(__url).get_response_field(NSPbr.get_resourcetype())
        __NSPbrs = []
        for json_NSPbr in __json_NSPbrs:
            __NSPbrs.append(NSPbr(json_NSPbr))
        return __NSPbrs



from nsbaseresource import NSBaseResource

__author__ = 'timl'


class NSResponderPolicyLabel(NSBaseResource):

    # Configuration for responder policy labels

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSResponderPolicyLabel, self).__init__()
        self.options = {
            'labelname': '',
            'policylabeltype': '',
            'numpol': '',
            'hits': '',
            'priority': '',
        }

        self.resourcetype = NSResponderPolicyLabel.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "responderpolicylabel"

    def set_labelname(self, labelname):
        """
        Name of the responder policy label.

        Default value: 0
        """
        self.options['labelname'] = labelname

    def get_labelname(self):
        """
        Name of the responder policy label.

        Default value: 0
        """
        return self.options['labelname']

    def set_policylabeltype(self, policylabeltype):
        """
        Policy label type  = "HTTP | OTHERTCP"

        Default value: 0
        """
        valid_types = ('HTTP', 'OTHERTCP')
        if policylabeltype and policylabeltype not in valid_types:
            raise ValueError("policylabeltype must be one of %s" %
                             ",".join(valid_types))
        self.options['policylabeltype'] = policylabeltype

    def get_policylabeltype(self):
        """
        Get the policylabeltype

        Default value: HTTP
        """
        return self.options['policylabeltype']

    def get_numpol(self):
        """
        Number of bound policies

        Default value: 0
        """
        return self.options['numpol']

    def get_hits(self):
        """
        Number of hits.

        Default value: 0
        """
        return self.options['hits']

    def set_priority(self, priority):
        """
        Set the priority for the responder policy label.

        Default value: 0
        """
        self.options['priority'] = priority

    def get_priority(self):
        """
        Priority of the responder policy label.

        Default value: 0
        """
        return self.options['priority']

    # Operations methods
    @staticmethod
    def get(nitro, responderpolicylabel):
        """
        Use this API to fetch responderpolicylabel resource of given name.
        """
        __responderpolicylabel = NSResponderPolicyLabel()
        __responderpolicylabel.set_labelname(responderpolicylabel.get_labelname())
        __responderpolicylabel.get_resource(nitro)
        return __responderpolicylabel

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured responderpolicylabel resources.
        """
        __url = nitro.get_url() + NSResponderPolicyLabel.get_resourcetype()
        __json_cspolicies = nitro.get(__url).get_response_field(NSResponderPolicyLabel.get_resourcetype())
        __responderpolicies = []
        for json_responderpolicylabel in __json_cspolicies:
            __responderpolicies.append(NSResponderPolicyLabel(json_responderpolicylabel))
        return __responderpolicies

    @staticmethod
    def add(nitro, responderpolicylabel):
        """
        Use this API to add responderpolicylabel.
        """
        __responderpolicylabel = NSResponderPolicyLabel()
        __responderpolicylabel.set_labelname(responderpolicylabel.get_labelname())
        __responderpolicylabel.set_policylabeltype(responderpolicylabel.get_policylabeltype())
        __responderpolicylabel.set_priority(responderpolicylabel.get_priority())
        return __responderpolicylabel.add_resource(nitro)

    @staticmethod
    def delete(nitro, responderpolicylabel):
        """
        Use this API to delete responderpolicylabel of a given name.
        """
        __responderpolicylabel = NSResponderPolicyLabel()
        __name = responderpolicylabel.get_labelname()
        __responderpolicylabel.set_labelname(__name)
        nsresponse = __responderpolicylabel.delete_resource(nitro, __name)
        return nsresponse

    @staticmethod
    def update(nitro, responderpolicylabel):
        """
        Use this API to update a responderpolicylabel of a given name.
        """
        __responderpolicylabel = NSResponderPolicyLabel()
        __responderpolicylabel.set_labelname(responderpolicylabel.get_labelname())
        __responderpolicylabel.set_policylabeltype(responderpolicylabel.get_policylabeltype())
        __responderpolicylabel.set_priority(responderpolicylabel.get_priority())
        return __responderpolicylabel.update_resource(nitro)


    # No unset functionality for now.

    # No rename functionality for now.



from nsbaseresource import NSBaseResource

__author__ = 'Aleksandar Topuzovic'


class NSVServer(NSBaseResource):

    # Configuration for Load Balancing Virtual Server resource.

    def __init__(self, json_data=None):
        """
        Supplied with json_data the object can be pre-filled
        """
        super(NSVServer, self).__init__()
        self.options = {
            "name": "",
            "insertvserveripport": "",
            "vipheader": "",
            "ip": "",
            "ipv46": "",
            "port": "",
            "range": "",
            "ipv6address": "",
            "ippattern": "",
            "ipmask": "",
            "servicetype": "",
            "value": "",
            "type": "",
            "state": "",
            "effectivestate": "",
            "status": "",
            "cachetype": "",
            "redirect": "",
            "precedence": "",
            "redirecturl": "",
            "authentication": "",
            "homepage": "",
            "dnsvservername": "",
            "domain": "",
            "rule": "",
            "policyname": "",
            "hits": "",
            "servicename": "",
            "weight": "",
            "cachevserver": "",
            "backupvserver": "",
            "priority": "",
            "clttimeout": "",
            "somethod": "",
            "sopersistence": "",
            "sopersistencetimeout": "",
            "sothreshold": "",
            "lbmethod": "",
            "hashlength": "",
            "dataoffset": "",
            "datalength": "",
            "netmask": "",
            "groupname": "",
            "m": "",
            "tosid": "",
            "persistencetype": "",
            "cookiedomain": "",
            "persistmask": "",
            "persistencebackup": "",
            "timeout": "",
            "cacheable": "",
            "pq": "",
            "sc": "",
            "sessionless": "",
            "url": "",
            "reuse": "",
            "destinationvserver": "",
            "via": "",
            "flags": "",
            "connfailover": "",
            "casesensitive": "",
            "redirectportrewrite": "",
            "downstateflush": "",
            "cookieipport": "",
            "vserverid": "",
            "version": "",
            "totalservices": "",
            "activeservices": "",
            "pushvserver": "",
        }

        self.resourcetype = NSVServer.get_resourcetype()

        if not (json_data is None):
            for key in json_data.keys():
                if key in self.options.keys():
                    self.options[key] = json_data[key]

    @staticmethod
    def get_resourcetype():
        return "vserver"

    # Read/write properties
    def set_name(self, name):
        """
        The name of the virtual server to be removed.
        Minimum length = 1
        """
        self.options['name'] = name

    def get_name(self):
        """
        The name of the virtual server to be removed.
        Minimum length = 1
        """
        return self.options['name']

    def set_backupvserver(self, backupvserver):
        """
        The name of the backup virtual server for this virtual server.
        Minimum length = 1
        """
        self.options['backupvserver'] = backupvserver

    def get_backupvserver(self):
        """
        The name of the backup virtual server for this virtual server.
        Minimum length = 1
        """
        return self.options['backupvserver']

    def set_redirecturl(self, redirecturl):
        """
        The URL where traffic is redirected if the virtual server in the system becomes unavailable.
        Minimum length = 1
        """
        self.options['redirecturl'] = redirecturl

    def get_redirecturl(self):
        """
        The URL where traffic is redirected if the virtual server in the system becomes unavailable.
        Minimum length = 1
        """
        return self.options['redirecturl']

    def set_cacheable(self, cacheable):
        """
        Use this option to specify whether a virtual server (used for load balancing or content
        switching) routes requests to the cache redirection virtual server before sending it to
        the configured servers.
        """
        self.options['cacheable'] = cacheable

    def get_cacheable(self):
        """
        Use this option to specify whether a virtual server (used for load balancing or content
        switching) routes requests to the cache redirection virtual server before sending it to
        the configured servers.
        """
        return self.options['cacheable']

    def set_clttimeout(self, clttimeout):
        """
        The timeout value in seconds for idle client connection.
        Minimum value = 0
        Maximum value = 31536000
        """
        self.options['clttimeout'] = clttimeout

    def get_clttimeout(self):
        """
        The timeout value in seconds for idle client connection.
        Minimum value = 0
        Maximum value = 31536000
        """
        return self.options['clttimeout']

    def set_somethod(self, somethod):
        """
        The spillover factor. The system will use this value to determine if it should send
        traffic to the backupvserver when the main virtual server reaches the spillover threshold.
        """
        self.options['somethod'] = somethod

    def get_somethod(self):
        """
        The spillover factor. The system will use this value to determine if it should send
        traffic to the backupvserver when the main virtual server reaches the spillover threshold.
        """
        return self.options['somethod']

    def set_sopersistence(self, sopersistence):
        """
        The state of the spillover persistence.
        Default value: DISABLED
        """
        self.options['sopersistence'] = sopersistence

    def get_sopersistence(self):
        """
        The state of the spillover persistence.
        Default value: DISABLED
        """
        return self.options['sopersistence']

    def set_sopersistencetimeout(self, sopersistencetimeout):
        """
        The spillover persistence entry timeout.
        Default value: 2
        Minimum value = 2
        Maximum value = 1440
        """
        self.options['sopersistencetimeout'] = sopersistencetimeout

    def get_sopersistencetimeout(self):
        """
        The spillover persistence entry timeout.
        Default value: 2
        Minimum value = 2
        Maximum value = 1440
        """
        return self.options['sopersistencetimeout']

    def set_sothreshold(self, sothreshold):
        """
        The spillver threshold value.
        Minimum value = 1
        Maximum value = 4294967294
        """
        self.options['sothreshold'] = sothreshold

    def get_sothreshold(self):
        """
        The spillver threshold value.
        Minimum value = 1
        Maximum value = 4294967294
        """
        return self.options['sothreshold']

    def set_pushvserver(self, pushvserver):
        """
        The lb vserver of type PUSH/SSL_PUSH to which server pushes the updates
        received on the client facing non-push lb vserver.
        Minimum length = 1
        """
        self.options['pushvserver'] = pushvserver

    def get_pushvserver(self):
        """
        The lb vserver of type PUSH/SSL_PUSH to which server pushes the updates
        received on the client facing non-push lb vserver.
        Minimum length = 1
        """
        return self.options['pushvserver']

    # Read only properties
    def get_insertvserveripport(self):
        """
        The virtual IP and port header insertion option for the vserver.
        """
        return self.options['insertvserveripport']

    def get_vipheader(self):
        """
        The name of the virtual IP and port header.
        """
        return self.options['vipheader']

    def get_ip(self):
        return self.options['ip']

    def get_ipv46(self):
        """
        The IP address of the virtual server.
        """
        return self.options['ipv46']

    def get_port(self):
        return self.options['port']

    def get_range(self):
        return self.options['range']

    def get_ipv6address(self):
        return self.options['ipv6address']

    def get_ippattern(self):
        """
        The IP pattern of the virtual server.
        """
        return self.options['ippattern']

    def get_ipmask(self):
        """
        The IP address mask of the virtual server.
        """
        return self.options['ipmask']

    def get_servicetype(self):
        return self.options['servicetype']

    def get_value(self):
        return self.options['value']

    def get_type(self):
        return self.options['type']

    def get_state(self):
        return self.options['state']

    def get_effectivestate(self):
        return self.options['effectivestate']

    def get_status(self):
        return self.options['status']

    def get_cachetype(self):
        return self.options['cachetype']

    def get_redirect(self):
        return self.options['redirect']

    def get_precedence(self):
        return self.options['precedence']

    def get_authentication(self):
        return self.options['authentication']

    def get_homepage(self):
        return self.options['homepage']

    def get_dnsvservername(self):
        return self.options['dnsvservername']

    def get_domain(self):
        return self.options['domain']

    def get_rule(self):
        return self.options['rule']

    def get_policyname(self):
        return self.options['policyname']

    def get_hits(self):
        return self.options['hits']

    def get_servicename(self):
        return self.options['servicename']

    def get_weight(self):
        return self.options['weight']

    def get_cachevserver(self):
        return self.options['cachevserver']

    def get_priority(self):
        return self.options['priority']

    def get_lbmethod(self):
        """
        lb method
        """
        return self.options['lbmethod']

    def get_hashlength(self):
        """
        max hash length
        """
        return self.options['hashlength']

    def get_dataoffset(self):
        """
        data offset
        """
        return self.options['dataoffset']

    def get_datalength(self):
        """
        data length
        """
        return self.options['datalength']

    def get_netmask(self):
        """
        hash netmask
        """
        return self.options['netmask']

    def get_groupname(self):
        """
        group name
        """
        return self.options['groupname']

    def get_m(self):
        """
        lb mode
        """
        return self.options['m']

    def get_tosid(self):
        """
        TOS ID
        """
        return self.options['tosid']

    def get_persistencetype(self):
        return self.options['persistencetype']

    def get_cookiedomain(self):
        """
        domain name
        """
        return self.options['cookiedomain']

    def get_persistmask(self):
        """
        persistence mask
        """
        return self.options['persistmask']

    def get_persistencebackup(self):
        """
        backup persistence type
        """
        return self.options['persistencebackup']

    def get_timeout(self):
        """
        timeout
        """
        return self.options['timeout']

    def get_pq(self):
        """
        number
        """
        return self.options['pq']

    def get_sc(self):
        """
        The state of SureConnect on this vserver.
        """
        return self.options['sc']

    def get_sessionless(self):
        """
        sessionless lb
        """
        return self.options['sessionless']

    def get_url(self):
        """
        URL for probe
        """
        return self.options['url']

    def get_reuse(self):
        """
        wtm
        """
        return self.options['reuse']

    def get_destinationvserver(self):
        """
        destination vserver
        """
        return self.options['destinationvserver']

    def get_via(self):
        """
        via
        """
        return self.options['via']

    def get_flags(self):
        """
        vserver flags
        """
        return self.options['flags']

    def get_connfailover(self):
        """
        connection failover
        """
        return self.options['connfailover']

    def get_casesensitive(self):
        """
        persistence type
        """
        return self.options['casesensitive']

    def get_redirectportrewrite(self):
        """
        port rewrite for ssl
        """
        return self.options['redirectportrewrite']

    def get_downstateflush(self):
        """
        Perform delayed clean up of connections on this vserver.
        """
        return self.options['downstateflush']

    def get_cookieipport(self):
        """
        Encryped Ip address and port of the service that is inserted into the
        set-cookie http header
        """
        return self.options['cookieipport']

    def get_vserverid(self):
        """
        Vserver Id
        """
        return self.options['vserverid']

    def get_version(self):
        """
        Cookie version
        """
        return self.options['version']

    def get_totalservices(self):
        """
        Total number of services bound to the vserver.
        """
        return self.options['totalservices']

    def get_activeservices(self):
        """
        Total number of active services bound to the vserver.
        """
        return self.options['activeservices']

    # Operations methods
    @staticmethod
    def disable(nitro, vserver):
        """
        Use this API to disable vserver.
        """
        __vserver = NSVServer()
        __vserver.set_name(vserver.get_name())
        return __vserver.perform_operation(nitro, "disable")

    @staticmethod
    def enable(nitro, vserver):
        """
        Use this API to enable vserver.
        """
        __vserver = NSVServer()
        __vserver.set_name(vserver.get_name())
        return __vserver.perform_operation(nitro, "enable")

    @staticmethod
    def get(nitro, vserver):
        """
        Use this API to fetch vserver resource of given name.
        """
        __vserver = NSVServer()
        __vserver.set_name(vserver.get_name())
        __vserver.get_resource(nitro)
        return __vserver

    @staticmethod
    def get_all(nitro):
        """
        Use this API to fetch all configured vserver resources.
        """
        __url = nitro.get_url() + NSVServer.get_resourcetype()
        __json_vservers = nitro.get(__url).get_response_field(NSVServer.get_resourcetype())
        __vservers = []
        for json_vserver in __json_vservers:
            __vservers.append(NSVServer(json_vserver))
        return __vservers

    @staticmethod
    def delete(nitro, vserver):
        """
        Use this API to delete vserver of a given name.
        """
        __vserver = NSVServer()
        __vserver.set_name(vserver.get_name())
        nsresponse = __vserver.delete_resource(nitro)
        return nsresponse

    @staticmethod
    def update(nitro, vserver):
        """
        Use this API to update vserver of a given name.
        """
        __vserver = NSVServer()
        __vserver.set_name(vserver.get_name())
        __vserver.set_backupvserver(vserver.get_backupvserver())
        __vserver.set_redirecturl(vserver.get_redirecturl())
        __vserver.set_cacheable(vserver.get_cacheable())
        __vserver.set_clttimeout(vserver.get_clttimeout())
        __vserver.set_somethod(vserver.get_somethod())
        __vserver.set_sopersistence(vserver.get_sopersistence())
        __vserver.set_sopersistencetimeout(vserver.get_sopersistencetimeout())
        __vserver.set_sothreshold(vserver.get_sothreshold())
        __vserver.set_pushvserver(vserver.get_pushvserver())
        return __vserver.update_resource(nitro)
